@@ -12,7 +12,7 @@ from sympy.logic import boolalg
 
 from expr_codegen.codes import sources_to_exprs
 from expr_codegen.expr import get_current_by_prefix, get_children, replace_exprs
-from expr_codegen.model import dag_start, dag_end, dag_middle
+from expr_codegen.model import dag_start, dag_end, dag_middle, _RESERVED_WORD_
 
 try:
     from pandas import DataFrame as _pd_DataFrame
@@ -232,6 +232,7 @@ class ExprTool:
 
         # 子表达式在前，原表式在最后
         exprs_dst, syms_dst = self.merge(date, asset, **exprs_src)
+        syms_dst = list(set(syms_dst) - _RESERVED_WORD_)
 
         # 提取公共表达式
         self.cse(exprs_dst, symbols_repl=numbered_symbols('_x_'), symbols_redu=exprs_src.keys())
@@ -298,22 +299,29 @@ class ExprTool:
         return code
 
 
-def _exec_code(code: str, df_input):
-    globals_ = {'df_input': df_input}
+@lru_cache(maxsize=64, typed=True)
+def _get_func_from_code(code: str):
+    globals_ = {}
     exec(code, globals_)
-    return globals_['df_output']
+    return globals_['main']
 
 
-def _exec_file(file, df_input):
-    with open(file, 'r', encoding='utf-8') as f:
-        code = f.read()
-        return _exec_code(code, df_input)
-
-
-def _exec_module(module: str, df_input):
+@lru_cache(maxsize=64, typed=True)
+def _get_func_from_module(module: str):
     """"可下断点调试"""
     m = __import__(module, fromlist=['*'])
-    return m.main(df_input)
+    logger.info(f'run module {m}')
+    return m.main
+
+
+@lru_cache(maxsize=64, typed=True)
+def _get_func_from_file(file: str):
+    file = pathlib.Path(file)
+    logger.info(f'run file "{file.absolute()}"')
+    with open(file, 'r', encoding='utf-8') as f:
+        globals_ = {}
+        exec(f.read(), globals_)
+        return globals_['main']
 
 
 _TOOL_ = ExprTool()
@@ -343,7 +351,7 @@ def codegen_exec(df: Optional[DataFrame],
     output_file: str| TextIOBase
         保存生成的目标代码到文件中
     run_file: bool or str
-        是否不生成脚本，直接运行代码。
+        是否不生成脚本，直接运行代码。注意：带缓存功能，多次调用不重复生成
         - 如果是True，会自动从output_file中读取代码
         - 如果是字符串，会自动从run_file中读取代码
         - 如果是模块名，会自动从模块中读取代码(可调试)
@@ -374,18 +382,13 @@ def codegen_exec(df: Optional[DataFrame],
     if df is not None:
         if run_file is True:
             assert output_file is not None, 'output_file is required'
-            output_file = pathlib.Path(output_file)
-            logger.info(f'run file "{output_file.absolute()}"')
-            return _exec_file(output_file, df)
+            return _get_func_from_file(output_file)(df)
         if run_file is not False:
             run_file = str(run_file)
             if run_file.endswith('.py'):
-                run_file = pathlib.Path(run_file)
-                logger.info(f'run file "{run_file.absolute()}"')
-                return _exec_file(run_file, df)
+                return _get_func_from_file(run_file)(df)
             else:
-                logger.info(f'run module "{run_file}"')
-                return _exec_module(run_file, df)  # 可断点调试
+                return _get_func_from_module(run_file)(df)  # 可断点调试
 
     # 此代码来自于sympy.var
     frame = inspect.currentframe().f_back
@@ -408,4 +411,4 @@ def codegen_exec(df: Optional[DataFrame],
     if df is None:
         return None
     else:
-        return _exec_code(code, df)
+        return _get_func_from_code(code)(df)
